/*
 *  common.h
 *
 *  Declarations for my Notes utilities written on C
 *
 *  Written by Ilya Evseev (evseev@csa.ru) at Oct 2000, Feb 2001
 */

#if defined(_WIN32)
#  if !defined(W32)
#    define W32  1       /* required by Notes */
#  endif
#  if !defined(__CONSOLE__) && !defined(__GUI__)
#    define __CONSOLE__ 1
#  endif
#  if defined(__CONSOLE__) && !defined(__WIN32_COMSOLE__)
#    define __WIN32_CONSOLE__ 1   /* my own macro */
#  endif
#endif 

#include <stdio.h>    /* stdio, stderr, fprintf() */
#include <stdlib.h>   /* atexit() */
#include <string.h>

#include <global.h>   /* NotesInitExtended(), NotesTerm(), MAXWORD, STATUS */
#include <srverr.h>   /* NOERROR, ERR(), ERR_xxx */
#include <nsfsearc.h> /* NSFSeach, ITEM_TABLE, SEARCH_MATCH, ... */

#define ACL_DEFAULT_NAME "-Default-"

#if defined(_WIN32) || defined(__MSDOS__)
#  define STDERR stdout   /* COMMAND.COM doesn't redirect stderr by ">" */
#else
#  define STDERR stderr
#endif

#ifndef ELEMS
#  define ELEMS(array)  (sizeof(array) / sizeof(array[0]))
#endif

#define EXIT_FAILURE 1
#define EXIT_USAGE   2

#define ERROR_TEXT_SIZE  256

#define NULL_TO_EMPTY(p)    ((p) ? (p) : "")

#define NULL_TO_ZEROINT(p)  ((p) ? atoi(p) : 0)


/*-------------------------------------------------------------
 *  Separator between parts of filepath
 */
#if defined (_WIN32) || defined (__OS2__)
#  define SLASH      '\\'
#  define SLASH_STR  "\\"
#elif defined(__linux__)
#  define SLASH      '/'
#  define SLASH_STR  "/"
#else
#  error "SLASH must be defined"
#endif


#if defined(__GNUC__)
extern char *strlwr(char *const s);
extern char *strupr(char *const s);
#endif

/*   NB:
 *  `status' variable must be declared before using following macros!
 */
#define IF_ERROR(__Hint, __Condition) \
        if ((status = notes_perror((__Condition), __Hint)) != NOERROR)

#define IF_OK(__Hint, __Condition) \
        if ((status = notes_perror((__Condition), __Hint)) == NOERROR)

#define RETURN_IF_ERROR(__Hint, __Condition) \
        IF_ERROR(__Hint, __Condition) \
            return status

#define GOTO_IF_ERROR(__Label, __Hint, __Condition) \
        IF_ERROR(__Hint, __Condition) \
            goto __Label

#define INIT_NOTES_SESSION()  \
        do {  \
            if ((status = NotesInitExtended(1, argv)) != NOERROR) {    \
                fprintf (STDERR, "\nERROR %d: cannot initialize Notes.\n", status);  \
                return EXIT_FAILURE;  \
            }  \
            atexit(terminate_notes_session);  \
        } while(0)

#if !defined(__cplusplus)
    typedef enum { false, true } bool;
#else
    extern "C" {
#endif

int notes_perror(STATUS status, const char *where);

STATUS do_action_for_each_database(char *filename, char *server,
    STATUS (*action)(HANDLE db, ITEM_TABLE *summary, void *args),
    void *action_args);

void terminate_notes_session(void);

#if defined(__cplusplus)
}  /*  extern "C" */
#endif


/* EOF */
